"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _react = require("react");

var React = _interopRequireWildcard(_react);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

var stubMethods = {
  shouldComponentUpdate: function shouldComponentUpdate() {
    return true;
  },
  render: function render() {
    throw new Error("render should be replaced with a wrapped render");
  },
  componentDidMount: function componentDidMount() {},
  getSnapshotBeforeUpdate: function getSnapshotBeforeUpdate() {
    return null;
  },
  componentDidUpdate: function componentDidUpdate() {}
};

var proxyLifecycleMethodsFor = function proxyLifecycleMethodsFor(self) {
  var original = {};

  // AppComponentProxy method definitions
  var proxyMethods = {
    shouldComponentUpdate: function shouldComponentUpdate(nextProps) {
      var props = self.props;
      var state = self.state;
      var appState = self.appState;

      try {
        self.props = this.props.props;
        self.state = this.props.state;
        self.appState = this.props.appState;
        return original.shouldComponentUpdate.call(self, nextProps.props, nextProps.state, nextProps.appState);
      } finally {
        self.props = props;
        self.state = state;
        self.appState = appState;
      }
    },
    render: function render() {
      self.appState = this.props.appState;
      self.setAppState = this.props.setAppState;
      return original.render.apply(self, arguments);
    },
    componentDidMount: function componentDidMount() {
      return original.componentDidMount.apply(self, arguments);
    },
    getSnapshotBeforeUpdate: function getSnapshotBeforeUpdate() {
      self.appState = this.props.appState;
      self.setAppState = this.props.setAppState;
      return original.getSnapshotBeforeUpdate.apply(self, arguments);
    },
    componentDidUpdate: function componentDidUpdate(prevProps, prevState, snapshot) {
      return original.componentDidUpdate.call(self, prevProps.props, prevProps.state, snapshot, prevProps.appState);
    }
  };

  // AppComponentProxy class
  var AppComponentProxy = function AppComponentProxy(props) {
    React.Component.apply(this, props);
  };
  AppComponentProxy.prototype = Object.create(React.Component.prototype);

  // Move methods from `self` to AppComponentProxy (and create `original` obj)
  for (var method in proxyMethods) {
    if (self[method]) {
      // Create `original` object
      original[method] = self[method];
      // Proxy method calls on proxy component to the instance (self) component
      AppComponentProxy.prototype[method] = proxyMethods[method];
      // Take proxied methods off `self` (as far as react sees)
      self[method] = stubMethods[method];
    }
  }

  return AppComponentProxy;
};

exports.default = proxyLifecycleMethodsFor;